# -*- coding: utf-8 -*-
"""
@author: Ce Song from Dalian University of Technology

Script for geometry optimization of repeat units.

Read in ‘Repeat_UNit_smiles.csv’ including the SMILES strings for all repeat units, then use the RDkit package to 
generate 3D coordinates for repeat untis which are requied for the geometry optimization conducted by RDKit.

The optimized structure of repeat unit * will be saved as './Repeat_Units/*.sdf'.

All repeat units are named in the format of {bisphenol name}_{di-halogen name} 
e.g., DHPZ_DBN repeat unit is constructed by the combination of DHPZ bisphenol monomer and DBN bisphenol monomer.  

"""

import os
import pandas as pd
from rdkit import Chem
from rdkit.Chem import AllChem


def Geo_Opt():
    
    RU_smiles = pd.read_csv('Repeat_UNit_smiles.csv')
    #RUs = RU_smiles.loc[:, 'Repeat_Unit'].to_list()
    RU_num = RU_smiles.shape[0]
    if not os.path.exists('./Repeat_Units'):
        os.mkdir('./Repeat_Units')


    for i in range(0, RU_num, 1):
    
        RU_name = RU_smiles.loc[i, 'Repeat_Unit']
        print('Repeat_Unit = %s' %i)
        print(RU_name)
        w = Chem.SDWriter('./Repeat_Units/%s.sdf' % RU_name)
        for j in [1,2,3,4]:
            smile = RU_smiles.loc[i, 'smile_%s' %j]
            mol=Chem.MolFromSmiles(smile)
            molecule = Chem.AddHs(mol)
            conformers = AllChem.EmbedMultipleConfs (molecule,numConfs = 20, randomSeed = 10, pruneRmsThresh=0.5,  numThreads =4)
            optimised_and_energies = AllChem.UFFOptimizeMoleculeConfs(molecule , maxIters=2000, numThreads =4, vdwThresh =100.0)
            EnergyDictionaryWithIDAsKey = {}
            for conformer in conformers :
                optimised , energy = optimised_and_energies[conformer]
                if optimised == 0:
                    EnergyDictionaryWithIDAsKey [ conformer ] = energy 
            lowestenergy = min( EnergyDictionaryWithIDAsKey.values () )
            for k, v in (EnergyDictionaryWithIDAsKey.items()):
                if v == lowestenergy:
                    lowestEnergyConformerID = k
            print(lowestEnergyConformerID)
            w.write(molecule, confId = lowestEnergyConformerID)


if __name__ == '__main__':
    
    Geo_Opt()


        
'''   
### Note ###

RDKit 2020.03.2.0 package may encounter difficulty in generating initial coordinates for "ADNP_DK" and "TOP_DK" repeat units 
from their SMILES strings.

Please try other RDkit versions or build initial coordinates for these repeat units manually or with other softwares.  

We provide an example of initial coordinates for "ADNP_DK" and "TOP_DK" repeat units, which is deposited in "./Repeat_Units/*.mol" 

Readers can use the following code to perform the geometry optimization on "ADNP_DK" and "TOP_DK" repeat units, which
employs the same parameter settings for the geometry optimization conducted above.



RUs_t = ['ADNP_DK', 'TOP_DK']
for RU_name in RUs_t:
    w = Chem.SDWriter('./Repeat_Units/%s.sdf' % RU_name)
    row_index = RU_smiles[RU_smiles.Repeat_Unit == RU_name].index.to_list()[0]
    for j in [1, 2, 3, 4]:
        mol = Chem.MolFromMolFile('./Repeat_Units/%s_%d.mol' % (RU_name, j), removeHs = False)
        opt = AllChem.UFFOptimizeMolecule(mol, maxIters=2000, vdwThresh =100.0)
        if opt == 0:
            w.write(mol)
        else: 
            print('More iterations for Geo_Opt are reqired')
    w.close()
'''            


