# -*- coding: utf-8 -*-
"""
@author: Ce Song from Dalian University of Technology

Script for calculating 211 descriptors of all repeat units with RDKit package.

Read in ‘Repeat_UNit_smiles.csv’ to get names for all repeat units, then use the RDkit package to 
calculate the descriptors of repeat untis based on the optimized structures from './Repeat_Units/*.sdf' files.

Descriptors of all repeat units are saved as 'RU_descriptors.csv' file, in which the columns are descriptors i.e., features, the rows are samples. 

"""

import pandas as pd
import os

from rdkit import Chem
from rdkit.Chem import Descriptors, rdMolDescriptors
from rdkit.ML.Descriptors import MoleculeDescriptors


def Cal_Des():
    
    RD_2d_descriptor_names = [x[0] for x in Descriptors._descList]
    RD_calculator = MoleculeDescriptors.MolecularDescriptorCalculator(RD_2d_descriptor_names)
    RD_3d_descriptor_names = ['PBF',
                              'PMI1', 'PMI2', 'PMI3',
                              'NPR1', 'NPR2', 
                              'RadiusOfGyration', 
                              'InertialShapeFactor',
                              'Eccentricity',
                              'Asphericity',
                              'SpherocityIndex',]

    RD_descriptor_names = RD_2d_descriptor_names + RD_3d_descriptor_names
    

    RU_smiles = pd.read_csv('Repeat_Unit_smiles.csv')
    all_RU_names = RU_smiles.loc[:, 'Repeat_Unit'].to_list()
    RU_num = RU_smiles.shape[0]

    RU_Descriptor_df = pd.DataFrame(index =all_RU_names, columns = ['Repeat_Unit']+RD_descriptor_names)

    for i in range(RU_num):
        print(i)
        RU_name = RU_smiles.loc[i, 'Repeat_Unit']
        RDdes = [0 for x in RD_descriptor_names]
        if os.access("./Repeat_Units/%s.sdf" % RU_name, os.F_OK):
            suppl = Chem.SDMolSupplier('./Repeat_Units/%s.sdf' % RU_name,removeHs = False)
            mols = [x for x in suppl if x is not None]
            #print('total RU num is %s' % len(mols))
            for mol in mols:
                rdkit_des_2d = list(RD_calculator.CalcDescriptors(mol))
                rdkit_des_3d = []
                rdkit_des_3d.append( rdMolDescriptors.CalcPBF(mol) )
                rdkit_des_3d.append( rdMolDescriptors.CalcPMI1(mol))
                rdkit_des_3d.append( rdMolDescriptors.CalcPMI2(mol))
                rdkit_des_3d.append( rdMolDescriptors.CalcPMI3(mol))
                rdkit_des_3d.append( rdMolDescriptors.CalcNPR1(mol))
                rdkit_des_3d.append( rdMolDescriptors.CalcNPR2(mol))
                rdkit_des_3d.append( rdMolDescriptors.CalcRadiusOfGyration(mol))
                rdkit_des_3d.append( rdMolDescriptors.CalcInertialShapeFactor(mol))
                rdkit_des_3d.append( rdMolDescriptors.CalcEccentricity(mol))
                rdkit_des_3d.append( rdMolDescriptors.CalcAsphericity(mol))
                rdkit_des_3d.append( rdMolDescriptors.CalcSpherocityIndex(mol))
                rdkit_des = rdkit_des_2d + rdkit_des_3d
                for m in range(len(rdkit_des)):
                    RDdes[m] += rdkit_des[m]
            RU_RDdes = [x/len(mols) for x in RDdes]
        
            if len(RD_descriptor_names) == len(RU_RDdes):
                for num in range(len(RU_RDdes)):
                    RU_Descriptor_df.loc[RU_name, RD_descriptor_names[num]] = RU_RDdes[num]
                RU_Descriptor_df.loc[RU_name,'Repeat_Unit'] = RU_name
                RU_Descriptor_df.to_csv('RU_descriptors.csv', index = 0)
            else:
                print('Bad Errors!!!')
            
        else:
            print('There is no %s.sdf file in Repeat_Units' % RU_name)
      

if __name__ == '__main__':
    
    Cal_Des()

